# Copyright 2022 Webull
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding=utf-8

from webullsdkcore.request import ApiRequest

class GetSnapshotRequest(ApiRequest):
    def __init__(self):
        ApiRequest.__init__(self, "/market-data/snapshot", version='v1', method="GET", query_params={})

    def set_symbols(self, symbols):
        if isinstance(symbols, str):
            self.add_query_param("symbols", symbols)
        elif isinstance(symbols, list):
            self.add_query_param("symbols", ",".join(symbols))
    
    def set_category(self, category):
        self.add_query_param("category", category)


# Copyright 2022 Webull
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding=utf-8

from webullsdkcore.common.easy_enum import EasyEnum


class ExchangeCode(EasyEnum):
    HKG = (1, 'Hong Kong Stock Exchange')
    NAS = (2, 'NASDAQ Capital Market')
    NYSE = (3, 'New York Stock Exchange')
    ASE = (4, 'NYSE MKT')
    PSE = (5, 'NYSE Archipelago Exchange')
    BAT = (6, 'BATS Exchanges')
    NMS = (7, "NASDAQ Global Market")
    NSQ = (8, "NASDAQ Global Select Market")
    OTC = (9, "Over The Counter Bulletin Board")
    PK = (10, "Pink Sheets")
    OPRA = (11, "Option Pricing Reporting Authority")
    CCC = (12, "Cryptocurrency")
    HKW = (13, "Hong Kong Warrant Exchange")



# Copyright 2022 Webull
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from webullsdkmdata.request.get_historical_bars_request import GetHistoricalBarsRequest
from webullsdkmdata.request.get_snapshot_request import GetSnapshotRequest


class MarketData:
    def __init__(self, api_client):
        self.client = api_client

    def get_history_bar(self, symbol, category, timespan, count='200'):
        """
        Returns to Instrument in the window aggregated data.
        According to the last N K-lines of the stock code, it supports various granularity K-lines such as m1 and m5.
        Currently, only the K-line with the previous weight is provided for the daily K-line and above,
        and only the un-weighted K-line is provided for the minute K.

        :param symbols: Securities code
        :param category: Security type, enumeration.
        :param timespan: K-line time granularity
        :param count: The number of lines: the default is 200, and the maximum limit is 1200
        """
        history_bar_request = GetHistoricalBarsRequest()
        history_bar_request.set_symbol(symbol)
        history_bar_request.set_category(category)
        history_bar_request.set_timespan(timespan)
        history_bar_request.set_count(count)
        response = self.client.get_response(history_bar_request)
        return response

    def get_snapshot(self, symbols, category):
        """
        Query the latest stock market snapshots in batches according to the stock code list.

        :param symbols: List of security codes; for example: single: 00700 multiple: 00700,00981;
        For each request,up to 100 symbols can be subscribed; Under the authority of Hong Kong stock BMP,
        a single request supports up to 20 symbols

        :param category: Security type, enumeration.
        """
        quote_request = GetSnapshotRequest()
        quote_request.set_symbols(symbols)
        quote_request.set_category(category)
        response = self.client.get_response(quote_request)
        return response



# Copyright 2022 Webull
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
from google.protobuf import json_format
from webullsdkmdata.quotes.grpc.response_code import ResponseCode


class Response(object):

    def __init__(self, result, data_obj):
        self.status_code = ResponseCode.OK if ResponseCode.SUCCEEDED == result.code else result.code
        self.msg = result.msg
        self.request_id = result.requestId
        self.path = result.path
        self._payload = None
        if data_obj and result.payload:
            data_obj.ParseFromString(result.payload)
            self._payload = data_obj
        if self.status_code != ResponseCode.OK:
            raise RuntimeError(self.status_code, self.msg)

    def json(self):
        if self._payload:
            return json.loads(json_format.MessageToJson(self._payload))



# Copyright 2022 Webull
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from webullsdkmdata.quotes.grpc.pb import quote_pb2
from webullsdkmdata.quotes.grpc.response import Response
from webullsdkmdata.request.grpc.get_historical_bars_request import GetHistoricalBarsRequest
from webullsdkmdata.request.grpc.get_quote_request import GetQuoteRequest
from webullsdkmdata.request.grpc.get_snapshot_request import GetSnapshotRequest
from webullsdkmdata.request.grpc.get_streaming_token_request import GetStreamingTokenRequest
from webullsdkmdata.request.grpc.quotes_subscribe_request import SubscribeRequest
from webullsdkmdata.request.grpc.quotes_unsubscribe_request import UnsubcribeRequest
from webullsdkmdata.request.grpc.get_tick_request import GetTickRequest


class MarketData:
    def __init__(self, grpc_client):
        self.client = grpc_client

    def get_token(self):
        """
        Obtain the token required by the market subscription interface, Subscribe and UnSubscribe require the token.
        Request the token before initiating the establishment of a persistent connection.
        """
        request = GetStreamingTokenRequest()
        result = self.client.get_response(request.get_path(), request.serialize())
        return Response(result, quote_pb2.TokenResponse())

    def create_subscription_rel(self, token, symbols, category, sub_types):
        """
       Real-time quotes unsubscribe interface is subscribed to real-time quotes pushes according to symbol and data type.

       :param token: Subscribe to the token returned by the pre-request,
        and the token needs to be a token that has already created a connection
       :param symbols: Such as: [AAPL,TSLA]
       :param category: Security type, enumeration
       :param sub_types: Unsubscribe data type, such as: [SNAPSHOT]、SubType Required when unsubscribe_all is empty
        or not true
       """
        request = SubscribeRequest(token, symbols, category, sub_types)
        result = self.client.get_response(request.get_path(), request.serialize())
        return Response(result, quote_pb2.SubscribeResponse())

    def remove_subscription_rel(self, token, symbols, category, sub_types, unsubscribe_all=False):
        """
        Real-time quotes unsubscribe interface is subscribed to real-time quotes pushes according to symbol and data type.
        When unsubscribing from the interface, you get no result returned if it succeeds, and Error is returned if it fails.

        :param token: Subscribe to the token returned by the pre-request,
         and the token needs to be a token that has already created a connection
        :param symbols: Such as: [AAPL,TSLA]
        :param category: Security type, enumeration
        :param sub_types: Unsubscribe data type, such as: [SNAPSHOT]、SubType Required when unsubscribe_all is empty
         or not true

        :param unsubscribe_all: boolean false (true means canceling all real-time market subscriptions.
         When unsubscribe_all is true, symbols, category, sub_types can be empty)
        """
        request = UnsubcribeRequest(token, symbols, category, sub_types, unsubscribe_all)
        result = self.client.get_response(request.get_path(), request.serialize())
        return Response(result, quote_pb2.SubscribeResponse())

    def get_history_bar(self, symbol, category, timespan, count='200'):
        """
        Returns to Instrument in the window aggregated data.
        According to the last N K-lines of the stock code, it supports various granularity K-lines such as m1 and m5.
        Currently, only the K-line with the previous weight is provided for the daily K-line and above,
        and only the un-weighted K-line is provided for the minute K.

        :param symbols: Securities code
        :param category: Security type, enumeration.
        :param timespan: K-line time granularity
        :param count: The number of lines: the default is 200, and the maximum limit is 1200
        """
        request = GetHistoricalBarsRequest(symbol, category, timespan, count)
        result = self.client.get_response(request.get_path(), request.serialize())
        return Response(result, quote_pb2.BarsResponse())

    def get_quote(self, symbol, category):
        """
        Query the depth quote of securities according to the stock code list.

        :param symbol: security codes
        :param category: Security type, enumeration.
        """
        request = GetQuoteRequest(symbol, category)
        result = self.client.get_response(request.get_path(), request.serialize())
        return Response(result, quote_pb2.QuoteResponse())

    def get_snapshot(self, symbols, category):
        """
        Query the latest stock market snapshots in batches according to the stock code list.

        :param symbols: List of security codes; for example: single: 00700 multiple: 00700,00981;
        For each request,up to 100 symbols can be subscribed; Under the authority of Hong Kong stock BMP,
        a single request supports up to 20 symbols

        :param category: Security type, enumeration.
        """
        request = GetSnapshotRequest(symbols, category)
        result = self.client.get_response(request.get_path(), request.serialize())
        return Response(result, quote_pb2.SnapshotResponse())

    def get_tick(self, symbol, category, count='30'):
        """
        Query tick-by-tick transaction of securities according to the stock code list.

        :param symbol: security codes
        :param category: Security type, enumeration.
        :param count: The number of lines: the default is 30, and the maximum limit is 1000
        """
        request = GetTickRequest(symbol, category, count)
        result = self.client.get_response(request.get_path(), request.serialize())
        return Response(result, quote_pb2.TickResponse())



# Copyright 2022 Webull
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding=utf-8

from webullsdkcore.common.easy_enum import EasyEnum


class Direction(EasyEnum):
    B = (1, 'Buy(stocks are traded at the first set bid price)')
    S = (2, 'Sell(stocks are traded at the first set ask price)')
    G = (3, 'Buy(stocks are traded at a higher price than the first set bid price)')
    L = (4, 'Sell(stocks are traded at a lower price than the first set ask price)')
    N = (5, 'Neutral')



# Copyright 2022 Webull
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding=utf-8

from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0bquote.proto\x12\x07openapi\"6\n\x11InstrumentRequest\x12\x0f\n\x07symbols\x18\x01 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\"9\n\x12InstrumentResponse\x12#\n\x06result\x18\x01 \x03(\x0b\x32\x13.openapi.Instrument\"j\n\nInstrument\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06symbol\x18\x02 \x01(\t\x12\x15\n\rinstrument_id\x18\x03 \x01(\t\x12\x15\n\rexchange_code\x18\x04 \x01(\t\x12\x10\n\x08\x63urrency\x18\x05 \x01(\t\"P\n\x0b\x42\x61rsRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\x10\n\x08timespan\x18\x03 \x01(\t\x12\r\n\x05\x63ount\x18\x04 \x01(\t\"S\n\x0c\x42\x61rsResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x15\n\rinstrument_id\x18\x02 \x01(\t\x12\x1c\n\x06result\x18\x03 \x03(\x0b\x32\x0c.openapi.Bar\"[\n\x03\x42\x61r\x12\x0c\n\x04time\x18\x01 \x01(\t\x12\x0c\n\x04open\x18\x02 \x01(\t\x12\r\n\x05\x63lose\x18\x03 \x01(\t\x12\x0c\n\x04high\x18\x04 \x01(\t\x12\x0b\n\x03low\x18\x05 \x01(\t\x12\x0e\n\x06volume\x18\x06 \x01(\t\"4\n\x0fSnapshotRequest\x12\x0f\n\x07symbols\x18\x01 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\"5\n\x10SnapshotResponse\x12!\n\x06result\x18\x01 \x03(\x0b\x32\x11.openapi.Snapshot\"\xc6\x01\n\x08Snapshot\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x15\n\rinstrument_id\x18\x02 \x01(\t\x12\x12\n\ntrade_time\x18\x03 \x01(\t\x12\r\n\x05price\x18\x04 \x01(\t\x12\x0c\n\x04open\x18\x05 \x01(\t\x12\x0c\n\x04high\x18\x06 \x01(\t\x12\x0b\n\x03low\x18\x07 \x01(\t\x12\x11\n\tpre_close\x18\x08 \x01(\t\x12\x0e\n\x06volume\x18\t \x01(\t\x12\x0e\n\x06\x63hange\x18\n \x01(\t\x12\x14\n\x0c\x63hange_ratio\x18\x0b \x01(\t\"\x1e\n\rTokenResponse\x12\r\n\x05token\x18\x01 \x01(\t\"p\n\x10SubscribeRequest\x12\r\n\x05token\x18\x01 \x01(\t\x12\x0f\n\x07symbols\x18\x02 \x03(\t\x12\x10\n\x08\x63\x61tegory\x18\x03 \x01(\t\x12\x11\n\tsub_types\x18\x04 \x03(\t\x12\x17\n\x0funsubscribe_all\x18\x05 \x01(\t\"\x13\n\x11SubscribeResponse\"0\n\x0cQuoteRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\"t\n\rQuoteResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x15\n\rinstrument_id\x18\x02 \x01(\t\x12\x1d\n\x04\x61sks\x18\x03 \x03(\x0b\x32\x0f.openapi.AskBid\x12\x1d\n\x04\x62ids\x18\x04 \x03(\x0b\x32\x0f.openapi.AskBid\"e\n\x06\x41skBid\x12\r\n\x05price\x18\x01 \x01(\t\x12\x0c\n\x04size\x18\x02 \x01(\t\x12\x1d\n\x05order\x18\x03 \x03(\x0b\x32\x0e.openapi.Order\x12\x1f\n\x06\x62roker\x18\x04 \x03(\x0b\x32\x0f.openapi.Broker\"#\n\x05Order\x12\x0c\n\x04mpid\x18\x01 \x01(\t\x12\x0c\n\x04size\x18\x02 \x01(\t\"#\n\x06\x42roker\x12\x0b\n\x03\x62id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\">\n\x0bTickRequest\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\r\n\x05\x63ount\x18\x03 \x01(\t\"T\n\x0cTickResponse\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x15\n\rinstrument_id\x18\x02 \x01(\t\x12\x1d\n\x06result\x18\x03 \x03(\x0b\x32\r.openapi.Tick\"A\n\x04Tick\x12\x0c\n\x04time\x18\x01 \x01(\t\x12\r\n\x05price\x18\x02 \x01(\t\x12\x0e\n\x06volume\x18\x03 \x01(\t\x12\x0c\n\x04side\x18\x04 \x01(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'quote_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _INSTRUMENTREQUEST._serialized_start=24
  _INSTRUMENTREQUEST._serialized_end=78
  _INSTRUMENTRESPONSE._serialized_start=80
  _INSTRUMENTRESPONSE._serialized_end=137
  _INSTRUMENT._serialized_start=139
  _INSTRUMENT._serialized_end=245
  _BARSREQUEST._serialized_start=247
  _BARSREQUEST._serialized_end=327
  _BARSRESPONSE._serialized_start=329
  _BARSRESPONSE._serialized_end=412
  _BAR._serialized_start=414
  _BAR._serialized_end=505
  _SNAPSHOTREQUEST._serialized_start=507
  _SNAPSHOTREQUEST._serialized_end=559
  _SNAPSHOTRESPONSE._serialized_start=561
  _SNAPSHOTRESPONSE._serialized_end=614
  _SNAPSHOT._serialized_start=617
  _SNAPSHOT._serialized_end=815
  _TOKENRESPONSE._serialized_start=817
  _TOKENRESPONSE._serialized_end=847
  _SUBSCRIBEREQUEST._serialized_start=849
  _SUBSCRIBEREQUEST._serialized_end=961
  _SUBSCRIBERESPONSE._serialized_start=963
  _SUBSCRIBERESPONSE._serialized_end=982
  _QUOTEREQUEST._serialized_start=984
  _QUOTEREQUEST._serialized_end=1032
  _QUOTERESPONSE._serialized_start=1034
  _QUOTERESPONSE._serialized_end=1150
  _ASKBID._serialized_start=1152
  _ASKBID._serialized_end=1253
  _ORDER._serialized_start=1255
  _ORDER._serialized_end=1290
  _BROKER._serialized_start=1292
  _BROKER._serialized_end=1327
  _TICKREQUEST._serialized_start=1329
  _TICKREQUEST._serialized_end=1391
  _TICKRESPONSE._serialized_start=1393
  _TICKRESPONSE._serialized_end=1477
  _TICK._serialized_start=1479
  _TICK._serialized_end=1544
# @@protoc_insertion_point(module_scope)



import os
from setuptools import setup, find_packages

PACKAGE = "webullsdkmdata"
DESCRIPTION = "The market data module of Webull Python SDK."
TOPDIR = os.path.dirname(__file__) or "."
VERSION = __import__(PACKAGE).__version__
AUTHOR = "Webull"
AUTHOR_EMAIL = ""
URL = ""
RD_CONTENT_TYPE = "text/markdown"
LICENSE = "Apache License 2.0"

with open("README.rst") as fp:
    LONG_DESCRIPTION = fp.read()

requires = [
    "webull-python-sdk-core==0.1.4",
    "webull-python-sdk-quotes-core==0.1.4"
]

setup_args = {
    'version': VERSION,
    'author': AUTHOR,
    'author_email': AUTHOR_EMAIL,
    'description': DESCRIPTION,
    'long_description_content_type': RD_CONTENT_TYPE,
    'license': LICENSE,
    'url': URL,
    'packages': find_packages(exclude=["tests*"]),
    'include_package_data': True,
    'platforms': 'any',
    'install_requires': requires
}

setup(name='webull-python-sdk-mdata', **setup_args)
